from os.path import abspath, dirname, exists, join

from configglue.pyschema import (BoolConfigOption, ConfigSection,
                                 DictConfigOption, IntConfigOption, Schema,
                                 SchemaConfigParser, StringConfigOption,
                                 schemaconfigglue, TupleConfigOption)
from launchpadlib.launchpad import STAGING_SERVICE_ROOT


class SchemaExtensionError(Exception): pass


class LeanKitSchema(Schema):
    debug = BoolConfigOption(default=False)
    extend = StringConfigOption()

    kanban = ConfigSection()
    kanban.host = StringConfigOption()
    kanban.username = StringConfigOption()
    kanban.password = StringConfigOption()

    lp = ConfigSection()
    lp.app_name = StringConfigOption(default='LeanKitKanban sync')
    lp.service_root = StringConfigOption(default=STAGING_SERVICE_ROOT)
    lp.cache_dir = StringConfigOption(default='/tmp/leankit')
    lp.access_token = StringConfigOption()
    lp.access_secret = StringConfigOption()
    lp.bug_url = StringConfigOption(default='https://bugs.launchpad.net/bugs/%s')
    lp.default_bug_sp = IntConfigOption(default=1)

    board = ConfigSection()
    board.id = StringConfigOption()
    board.default_lane_id = StringConfigOption()
    board.default_type = StringConfigOption(default='Task')
    board.projects = TupleConfigOption()

    mappings = ConfigSection()
    mappings.users = DictConfigOption()
    mappings.users = DictConfigOption()
    mappings.priorities = DictConfigOption(default={# LP priority:KB priority id
                                                    'Undecided': 0,
                                                    'Wishlist': 0,
                                                    'Low': 0,
                                                    'Medium': 1,
                                                    'High': 2,
                                                    'Critical': 3})
    mappings.types = DictConfigOption(default={# LP tag:KB type id
                                               'kb-defect': 'Defect',
                                               'kb-feature': 'Feature',
                                               'kb-improvement': 'Improvement',
                                               'kb-task': 'Task'})
    mappings.lanes = DictConfigOption(default={# KB lane ID: LP bug state
                                               # eg: 123456: In Progress
                                               })
    mappings.projects = DictConfigOption(default={# KB lane ID: LP project
                                                  })
    mappings.titles = DictConfigOption(default={# LP project: KB title tag
                                                })

    @staticmethod
    def get_config(files):
        config = SchemaConfigParser(LeanKitSchema())
        config.read(files)

        op, opts, args = schemaconfigglue(config)

        is_valid, reasons = config.is_valid(report=True)
        if not is_valid:
            op.error(reasons[0])

        return config


def load_app_config():
    """Load the application configuration files.

    Also handles the 'extend' schema option. Returns the loaded config.
    """
    configdir = abspath(dirname(__file__))
    config_file = join(configdir, 'config.ini')
    config = LeanKitSchema.get_config([config_file])
    load_config_extensions(config, configdir)
    return config

def load_config_extensions(config, configdir):
    extend = config.values('__main__')['extend']
    if extend is '':
        return

    extension = join(configdir, extend)
    if not exists(extension):
        raise SchemaExtensionError(
            "Unable to find config file extension '{0}'".format(extension))
    config.read(extension)
