import json
import urllib2


class Card(object):

    def __init__(self, external_system_url, type_name, priority, lane_id,
                 assigned_user_id, size, title):
        self.external_system_url = external_system_url
        self.type_name = type_name
        self.priority = priority
        self.lane_id = lane_id
        self.assigned_user_id = assigned_user_id
        self.size = size
        self.title = title

    def is_bug_card(self, config):
        bug_url = config.values('lp')['bug_url'] % ''
        return self.external_system_url.startswith(bug_url)


class LeanKitBase(object):
    _board = None
    _identifiers = None

    @property
    def debug(self):
        return self.config.values('__main__')['debug']

    @property
    def board(self):
        if self._board is None:
            self._board = self.do_request('/Kanban/Api/Boards/%s' % \
                                          self.config.values('board')['id'])
        return self._board

    def all_lanes(self):
        board = self.board['ReplyData'][0]
        for lane in board['Lanes']:
            yield lane
        for lane in board['Backlog']:
            yield lane

    def all_cards(self):
        for lane in self.all_lanes():
            for card in lane['Cards']:
                yield Card(external_system_url=card['ExternalSystemUrl'],
                           type_name=card['TypeName'],
                           priority=card['Priority'],
                           lane_id=card['LaneId'],
                           assigned_user_id=card['AssignedUserId'],
                           size=card['Size'],
                           title=card['Title'])

    @property
    def identifiers(self):
        if self._identifiers is None:
            self._identifiers = self.do_request( \
                '/Kanban/Api/Board/%s/GetBoardIdentifiers' \
                % self.config.values('board')['id'])
        return self._identifiers

    def __init__(self, config, lp):
        self.config = config
        self.kb_config = config.values('kanban')
        self.lp = lp
        self.init_urllib()

    def init_urllib(self):
        pwmgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        pwmgr.add_password(None, self.kb_config['host'] + '/',
                           self.kb_config['username'],
                           self.kb_config['password'])
        handler = urllib2.HTTPBasicAuthHandler(pwmgr)
        opener = urllib2.build_opener(handler)
        urllib2.install_opener(opener)

    def do_request(self, path, data=None):
        debug = self.config.values('__main__')['debug']
        if debug:
            print "Request:"
            print self.kb_config['host'] + path
        headers = {}
        if data is not None:
            headers = {'Content-type': 'application/json'}
            data = json.dumps(data)
            if debug:
                print data
        req = urllib2.Request(self.kb_config['host'] + path, data, headers)
        resp = urllib2.urlopen(req)
        json_resp = json.load(resp)
        #if debug:
        #    print "Response:"
        #    print json_resp
        return json_resp
